local _G = getfenv(0)
local VERSION = tonumber(("$Revision: 42008 $"):match("%d+"))
local L = AceLibrary("AceLocale-2.2"):new("AuldLangSyne_Fu")

local tablet = AceLibrary("Tablet-2.0")
local BC = AceLibrary("Babble-Class-2.2")

local AuldLangSyne = AuldLangSyne
local Fu = AuldLangSyne:NewModule("Fu", "AceHook-2.1", "AceEvent-2.0", "FuBarPlugin-2.0")
if AuldLangSyne.revision < VERSION then
	AuldLangSyne.version = "r" .. VERSION
	AuldLangSyne.revision = VERSION
	AuldLangSyne.date = ("$Date: 2007-06-28 05:29:45 -0400 (Thu, 28 Jun 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local new, del, newHash, newSet, deepCopy, deepDel = AuldLangSyne.new, AuldLangSyne.del, AuldLangSyne.newHash, AuldLangSyne.newSet, AuldLangSyne.deepCopy, AuldLangSyne.deepDel

Fu.name = "AuldLangSyneFu"
Fu.title = "AuldLangSyneFu"
Fu.clickableTooltip = true
Fu.hasIcon = "Interface\\FriendsFrame\\FriendsFrameScrollIcon"
Fu.OnMenuRequest = AuldLangSyne.menu

function Fu:OnInitialize(name)
	self.db = AuldLangSyne:AcquireDBNamespace("Fu")
	AuldLangSyne:RegisterDefaults("Fu", "profile", {
		friends = {
			show = true,
			offline = false,
			columns = {
				name = true,
				level = true,
				class = false,
				zone = true,
				status = true,
			},
			colors = {
				name = 'class',
				level = 'level',
				class = 'class',
				zone = 'none',
			},
		},
		guild = {
			show = true,
			offline = false,
			motd = true,
			columns = {
				name = true,
				level = true,
				class = false,
				zone = true,
				status = true,
				rank = true,
			},
			colors = {
				name = 'class',
				level = 'level',
				class = 'class',
				zone = 'none',
				rank = 'none',
			},
		},
	})
	
	local function makeGetter(kind, option)
		return function(key) return self.db.profile[kind][option][key] end
	end
	local function makeSetter(kind, option)
		return function(key, t) self.db.profile[kind][option][key] = t; self:Update() end
	end
	local function makeColumnsGroup(kind)
		group = {
			type = "group",
			name = "Columns", desc = "Columns",
			pass = true,
			get = makeGetter(kind, 'columns'),
			set = makeSetter(kind, 'columns'),
			args = {},
		}
		for column in pairs(self.db.profile[kind].columns) do
			group.args[column] = {type="toggle", name=column, desc=column,}
		end
		return group
	end
	local colorChoices = {'class', 'level', 'none'}
	local function makeColorsGroup(kind)
		group = {
			type = "group",
			name = "Colors", desc = "Colors",
			pass = true,
			get = makeGetter(kind, 'colors'),
			set = makeSetter(kind, 'colors'),
			args = {},
		}
		for column in pairs(self.db.profile[kind].colors) do
			group.args[column] = {type='text', name=column, desc=column, validate=colorChoices,}
		end
		return group
	end
	self.menu = {
		handler = Fu,
		name = "FuBar", type = "group",
		desc = "FuBar plugin options",
		args = {
			friends = {
				type = "group",
				name = "Friends", desc = "Friends",
				args = {
					show = {
						type = 'toggle',
						name = "Show", desc = "Show",
						get = function() return self.db.profile.friends.show end,
						set = function(t) self.db.profile.friends.show = t; self:Update() end,
						order = 1,
					},
					offline = {
						type = 'toggle',
						name = 'Offline', desc = "Offline",
						get = function() return self.db.profile.friends.offline end,
						set = function(t) self.db.profile.friends.offline = t; self:UpdateTooltip() end,
					},
					columns = makeColumnsGroup('friends'),
					colors = makeColorsGroup('friends'),
				},
			},
			guild = {
				type = "group",
				name = "Guild", desc = "Guild",
				args = {
					show = {
						type = 'toggle',
						name = "Show", desc = "Show",
						get = function() return self.db.profile.guild.show end,
						set = function(t) self.db.profile.guild.show = t; self:Update() end,
						order = 1,
					},
					offline = {
						type = 'toggle',
						name = 'Offline', desc = "Offline",
						get = function() return self.db.profile.guild.offline end,
						set = function(t) self.db.profile.guild.offline = t; self:UpdateTooltip() end,
					},
					motd = {
						type = 'toggle',
						name = "MotD", desc = "Message of the Day",
						get = function() return self.db.profile.guild.motd end,
						set = function(t) self.db.profile.guild.motd = t; self:Update() end,
					},
					columns = makeColumnsGroup('guild'),
					colors = makeColorsGroup('guild'),
				},
			}
		},
	}
	
	self.friends = new()
	self.guild = new()
	self.levels = new()
	self.classes = new()
	self.areas = new()
	self.status = new()
	self.connected = new()
	self.ranks = new()
	self.numFriends = 0
	self.numFriendsOnline = 0
end

function Fu:OnEnable(first)
	self:RegisterEvent("FRIENDLIST_UPDATE", "Update")
	self:SecureHook("FriendsList_Update", "Update")
	self:RegisterEvent("GUILD_ROSTER_UPDATE", "Update")
end

------------------------------------------------------------------------
-- FuBar methods
------------------------------------------------------------------------

function Fu:OnDataUpdate()
	del(self.friends)
	del(self.guild)
	del(self.levels)
	del(self.classes)
	del(self.areas)
	del(self.status)
	del(self.connected)
	del(self.ranks)
	self.friends = new()
	self.guild = new()
	self.levels = new()
	self.classes = new()
	self.areas = new()
	self.status = new()
	self.connected = new()
	self.ranks = new()
	
	if self.db.profile.friends.show then
		self:UpdateFriends()
	end
	if self.db.profile.guild.show then
		self:UpdateGuild()
	end
end

function Fu:UpdateFriends()
	local numFriends = GetNumFriends()
	local numFriendsOnline = 0
	for i=1, numFriends do
		local name, level, class, area, connected, status = GetFriendInfo(i)
		
		if name then
			table.insert(self.friends, name)
			if connected then
				numFriendsOnline = numFriendsOnline + 1
				self.levels[name] = level
				self.classes[name] = class
				self.areas[name] = area
				self.status[name] = status
			end
			self.connected[name] = connected
		end
	end
	self.numFriends = numFriends
	self.numFriendsOnline = numFriendsOnline
end

function Fu:UpdateGuild()
	self.numGuild = GetNumGuildMembers(true)
	--self.numGuildOnline = GetNumGuildMembers(false)
	self.numGuildOnline = 0
	for i=1, self.numGuild do
		name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i)
		if name then
			if online then
				self.numGuildOnline = self.numGuildOnline + 1
			end
			table.insert(self.guild, name)
			self.ranks[name] = rank
			self.levels[name] = level
			self.classes[name] = class
			self.areas[name] = zone
			self.connected[name] = online
			self.status[name] = status
		end
	end
end

function Fu:OnTextUpdate()
	if self.db.profile.friends.show or self.db.profile.guild.show then
		t = ''
		if self.db.profile.friends.show then
			t = t..self.numFriendsOnline..'/'..self.numFriends
		end
		if self.db.profile.friends.show and self.db.profile.guild.show then
			t = t..' '
		end
		if self.db.profile.guild.show then
			t = t..self.numGuildOnline..'/'..self.numGuild
		end
	end
	self:SetText(t)
end

do
	local function showNotes(name)
		local notes = AuldLangSyne:HasModule("Note") and AuldLangSyne:GetModule("Note")
		if notes then
			GameTooltip:SetOwner(_G.this, 'ANCHOR_RIGHT', 7, -18)
			GameTooltip:ClearLines()
			if notes.db.realm.friend[name] then
				GameTooltip:AddLine(notes.db.realm.friend[name], 1, 0.5, 0, true)
			end
			if notes.db.realm.guild[name] then
				GameTooltip:AddLine(notes.db.realm.guild[name], 0, 1, 0, true)
			end
			GameTooltip:AddLine("Click to edit", 0.5, 0.5, 0.5)
			GameTooltip:Show()
		end
	end
	local function editNote(name, type)
		local notes = AuldLangSyne:HasModule("Note") and AuldLangSyne:GetModule("Note")
		if notes then
			if notes.editFrame:IsVisible() and notes.editFrame.name == name then
				notes.editFrame.name = ""
				notes.editFrame.type = ""
				notes.editFrame:Hide()
			else
				notes.editFrame.name = name
				notes.editFrame.type = type

				notes.editFrame.editbox:SetText(notes.db.realm[type][name] or '')
				notes.editFrame.edit_text:SetText(format(AceLibrary("AceLocale-2.2"):new("AuldLangSyne_Note")["Editing note for %s"], name))
				notes.editFrame:Show()
			end
		end
	end
	function Fu:OnTooltipUpdate()
		local info = AuldLangSyne:HasModule("Info") and AuldLangSyne:GetModule("Info")
		
		local fopts = self.db.profile.friends
		if fopts.show then
			local friendcols = new()
			if fopts.columns.name then table.insert(friendcols, 'Name') end
			if fopts.columns.level then table.insert(friendcols, 'Level') end
			if fopts.columns.class then table.insert(friendcols, 'Class') end
			if fopts.columns.zone then table.insert(friendcols, 'Zone') end
			if fopts.columns.status then table.insert(friendcols, 'Status') end
			local header = new()
			for i,col in pairs(friendcols) do
				if i==1 then
					header['text'] = col
				else
					header['text'..i] = col
				end
			end
			
			local cat = tablet:AddCategory('id', 'friends', 'columns', #friendcols, 'text', 'Friends', 'justify', 'CENTER')
			cat:AddLine(header)
			local tmp = new()
			for _,name in pairs(self.friends) do
				if fopts.offline or self.connected[name] then
					tmp.name = name
					if self.connected[name] then
						tmp.level = self.levels[name]
						tmp.class = self.classes[name]
						tmp.zone = self.areas[name]
						tmp.status = self.status[name]
					else
						local thisinfo = info and info.db.realm[name]
						tmp.level = thisinfo and thisinfo.level or '?'
						tmp.class = thisinfo and thisinfo.class or '?'
						tmp.zone = ''
						tmp.status = thisinfo and info:HowLongAgo(thisinfo.updated) or '?'
					end
					line = new()
					for i,col in pairs(friendcols) do
						col = string.lower(col)
						local key, r, g, b
						if i==1 then key = 'text' else key = 'text'..i end
						if fopts.colors[col] == 'class' then
							r,g,b = BC:GetColor(tmp.class)
						elseif fopts.colors[col] == 'level' then
							local c = GetDifficultyColor(tmp.level)
							r,g,b = c.r, c.g, c.b
						else
							if self.connected[name] then
								r,g,b = 1,1,1
							else
								r,g,b = 0.5,0.5,0.5
							end
						end
						line[key] = tmp[col]
						line[key..'R'], line[key..'G'], line[key..'B'] = r, g, b
					end
					line['func'] = editNote; line['arg1'] = name; line['arg2'] = 'friend'
					line['onEnterFunc'] = showNotes; line['onEnterArg1'] = name
					line['onLeaveFunc'] = GameTooltip.Hide; line['onLeaveArg1'] = GameTooltip
					cat:AddLine(line)
					del(line)
				end
			end
			del(tmp)
			del(header)
			del(friendcols)
		end
		
		local gopts = self.db.profile.guild
		if gopts.show then
			local guildcols = new()
			if gopts.columns.name then table.insert(guildcols, 'Name') end
			if gopts.columns.level then table.insert(guildcols, 'Level') end
			if gopts.columns.class then table.insert(guildcols, 'Class') end
			if gopts.columns.zone then table.insert(guildcols, 'Zone') end
			if gopts.columns.rank then table.insert(guildcols, 'Rank') end
			if gopts.columns.status then table.insert(guildcols, 'Status') end
			local header = new()
			for i,col in pairs(guildcols) do
				if i==1 then
					header['text'] = col
				else
					header['text'..i] = col
				end
			end
			
			if gopts.motd then
				cat = tablet:AddCategory('id', 'motd', 'text', 'MotD')
				cat:AddLine('text', GetGuildRosterMOTD(), 'wrap', true)
			end
			
			cat = tablet:AddCategory('id', 'guild', 'columns', #guildcols, 'text', 'Guild')
			cat:AddLine(header)
			local tmp = new()
			for _,name in pairs(self.guild) do
				if gopts.offline or self.connected[name] then
					tmp.name = name
					tmp.level = self.levels[name]
					tmp.class = self.classes[name]
					tmp.zone = self.areas[name]
					tmp.status = self.status[name]
					tmp.rank = self.ranks[name]
					line = new()
					for i,col in pairs(guildcols) do
						col = string.lower(col)
						local key, r, g, b
						if i==1 then key = 'text' else key = 'text'..i end
						if gopts.colors[col] == 'class' then
							r,g,b = BC:GetColor(tmp.class)
						elseif gopts.colors[col] == 'level' then
							local c = GetDifficultyColor(tmp.level)
							r,g,b = c.r, c.g, c.b
						else
							if self.connected[name] then
								r,g,b = 1,1,1
							else
								r,g,b = 0.5,0.5,0.5
							end
						end
						line[key] = tmp[col]
						line[key..'R'], line[key..'G'], line[key..'B'] = r, g, b
					end
					line['func'] = editNote; line['arg1'] = name; line['arg2'] = 'guild'
					line['onEnterFunc'] = showNotes; line['onEnterArg1'] = name
					line['onLeaveFunc'] = GameTooltip.Hide; line['onLeaveArg1'] = GameTooltip
					cat:AddLine(line)
					del(line)
				end
			end
			del(tmp)
			del(header)
			del(guildcols)
		end
	end
end
